@@ -47,8 +47,9 @@ from mercurial import mail
 from urllib2 import AbstractBasicAuthHandler, AbstractDigestAuthHandler
 
 # mercurial.demandimport incompatibility workaround, various keyring
-# backends fail due to it.
-from mercurial.demandimport import ignore
+# backends fail due to it. This should not be necessary once next
+# workaround is present, but to be safe I preserve this setting.
+from mercurial import demandimport
 for blocked_module in [
     "gobject._gobject",
     "configparser",
@@ -63,11 +64,23 @@ for blocked_module in [
     "atom.http_interface",
     "atom.service",
     "atom.token_store",
+    "ctypes",
+    "secretstorage.exceptions",
     ]:
-    if blocked_module not in ignore:
-        ignore.append(blocked_module)
+    if blocked_module not in demandimport.ignore:
+        demandimport.ignore.append(blocked_module)
+
+# Temporarily disable demandimport to make the need of extending
+# list above less likely.
+if __import__ == demandimport._import:
+    demandimport.disable()
+    try:
+        import keyring
+    finally:
+        demandimport.enable()
+else:
+    import keyring
 
-import keyring
 from urlparse import urlparse
 import urllib2
 import smtplib, socket
